######################################################################
## STEP 7
## Describing segments
######################################################################

suppressWarnings(RNGversion("3.5"))

######################################################################
## 7.2 Using visualisation to describe market segments
######################################################################

## See Section 5.5.4.
library("flexclust")
data("vacmot", package = "flexclust")
set.seed(1234)
vacmot.k38 <- stepcclust(vacmot, k = 3:8, method = "neuralgas",
                         nrep = 20, save.data = TRUE, verbose = FALSE)
vacmot.k38 <- relabel(vacmot.k38)
vacmot.k6 <- vacmot.k38[["6"]]

C6 <- clusters(vacmot.k6)
table(C6)

vacmotdesc$C6 <- as.factor(C6)

## 7.2.1 Visualising nominal and ordinal descriptor variables

C6.Gender <- with(vacmotdesc, table("Segment number" = C6, Gender))
C6.Gender

opar <- par(mfrow = 2:1, mar = c(2, 4, 2, 2))
barplot(t(C6.Gender)[2:1, ], beside = FALSE,
        ylab = "Number of segment members",
        col = c("grey20", "grey80"), ylim = c(0, 400))
legend("topright", c("Male", "Female"), col = c("grey80", "grey20"),
       pch = 16)
mosaicplot(C6.Gender, shade = FALSE, main = "")
par(opar)

mosaicplot(C6.Gender, shade = TRUE, main = "")
mosaicplot(with(vacmotdesc, table(C6, Income2)), shade = TRUE,
           main = "", xlab = "Segment number")
mosaicplot(with(vacmotdesc, table(C6, Obligation2)), shade = TRUE,
           main = "", xlab = "Segment number")

## 7.2.2 Visualising metric descriptor variables

library("lattice")
histogram(~ Age | factor(paste("Segment", C6)), 
          data = vacmotdesc, as.table = TRUE)
histogram(~ Obligation | factor(paste("Segment", C6)), 
          data = vacmotdesc, as.table = TRUE)

boxplot(Age ~ C6, data = vacmotdesc,
        xlab = "Segment number", ylab = "Age")
boxplot(Obligation ~ C6, data = vacmotdesc,
        varwidth = TRUE,  notch = TRUE,
        xlab = "Segment number", ylab = "Moral obligation")

## See Section 5.5.4.2 Segment level stability across solutions.
slsaplot(vacmot.k38, nodecol = vacmotdesc$Obligation)

######################################################################
## 7.3 Testing for segment differences in descriptor variables
######################################################################

chisq.test(C6.Gender)
chisq.test(with(vacmotdesc, table(C6, Obligation2)))

C6.moblig <- with(vacmotdesc, tapply(Obligation, C6, mean))
C6.moblig

aov1 <- aov(Obligation ~ C6, data = vacmotdesc)
summary(aov1)

plot(TukeyHSD(aov1))
mtext("Pairs of segments", side = 2, line = 3)

with(vacmotdesc, pairwise.t.test(Obligation, C6))

C6.age <- with(vacmotdesc, tapply(Age, C6, mean))
C6.age
summary(aov(Age ~ C6, data = vacmotdesc))

######################################################################
## 7.4 Predicting segments from descriptor variables
######################################################################

lm(Age ~ C6 - 1, data = vacmotdesc)
lm(Age ~ C6, data = vacmotdesc)

## 7.4.1 Binary logistic regression

f <- I(C6 == 3) ~ Age + Obligation2
model.C63 <- glm(f, data = vacmotdesc, family = binomial())
model.C63

library("effects")
plot(allEffects(mod = model.C63))

summary(model.C63)  

library("car")
Anova(model.C63)

step.model.C63 <- step(glm(I(C6 == 3) ~ ., 
  data = na.omit(vacmotdesc), family = binomial()),
  trace = 0)
summary(step.model.C63)  

opar <- par(mfrow = c(1, 2))
prob.C63 <- predict(model.C63, type = "response")
boxplot(prob.C63 ~ I(C6 == 3), data = vacmotdesc, 
  ylim = 0:1, main = "", ylab = "Predicted probability")
prob.step.C63 <- predict(step.model.C63, type = "response")
boxplot(prob.step.C63 ~ I(C6 == 3), 
  data = na.omit(vacmotdesc), ylim = 0:1, 
  main = "", ylab = "Predicted probability")
par(opar)

## 7.4.2 Multinomial logistic regression

library("nnet")
model.C6 <- multinom(C6 ~ Age + Obligation2, 
  data = vacmotdesc, trace = 0)
model.C6  

summary(model.C6)

Anova(model.C6)

opar <- par(mfrow = c(1, 2))
pred.class.C6 <- predict(model.C6)
plot(table(observed = vacmotdesc$C6, 
  predicted = pred.class.C6), main = "")
pred.prob.C6 <- predict(model.C6, type = "prob")
predicted <- data.frame(prob = as.vector(pred.prob.C6),
  observed = C6, predicted = rep(1:6, each = length(C6)))
boxplot(prob ~ predicted,
  data = subset(predicted, observed == 6))
par(opar)

plot(allEffects(mod = model.C6), layout = c(3, 2))

## 7.4.3 Tree-based methods

set.seed(1234)
library("partykit")
tree63 <- ctree(factor(C6 == 3) ~ ., data = vacmotdesc)
tree63
plot(tree63)

tree66 <- ctree(factor(C6 == 6) ~ ., data = vacmotdesc, 
  control = ctree_control(minbucket = 100, mincriterion = 0.99))
plot(tree66)  

set.seed(4)
tree6 <- ctree(C6 ~ ., data = vacmotdesc)
tree6
plot(tree6)


